/*
 * This file is part of the QIMSI QL interface support software
 *
 * Copyright (C) 2023 Peter Graf
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/* Linker-Commandfile for SDRAM-less QIMSI and m68k-coff GNU Linker */ 

OUTPUT_FORMAT(binary)
/* OUTPUT_FORMAT(srec) */

ENTRY(start)

MEMORY
{
  ROM :  org = 0x00000000 , l = 32K   /* QIMSI FPGA ROMRAM    */
  VRAM : org = 0x00020000 , l = 32K   /* SCREEN 0             */
  RAM :  org = 0x00019000 , l = 12K   /* QIMSI FPGA RAM       */
}

/* Section Names
     .vector  Vectortable
     .text    Instructions and read-only data
     .data    initialized writable assembler data
     .bss     global and static variables
     .stack   System Stack
     
   Expressions
     *(section) Refer to all input files
     >region    Assign to region defined with MEMORY 
*/

SECTIONS
{
  .vector :
    { _vector_start = .; *(.vector); . = ALIGN(4); _vector_end = .; } > ROM

  .text : 
    { _text_start = .; *(.text); . = ALIGN(4); _text_end = .; } > ROM

  .data :
    { _data_start = .; *(.data); . = ALIGN(4); _data_end = .; } > ROM
  
  .bss (NOLOAD) :          
    { _bss_start = .; *(.bss); *(COMMON); . = ALIGN(4); _bss_end = .; } > RAM
    
  .stack (NOLOAD) :
    { _stack_start = .; . += 768; *(.stack); _stack_end = .; } > RAM
}
